@@ -1,5 +1,5 @@
 # This file is part of the pyMOR project (https://www.pymor.org).
-# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# Copyright pyMOR developers and contributors. All rights reserved.
 # License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 from pymor.algorithms.gram_schmidt import gram_schmidt
@@ -91,7 +91,7 @@ def estimate_image(operators=(), vectors=(),
             try:
                 rules.apply(v)
             except NoMatchingRuleError as e:
-                raise ImageCollectionError(e.obj)
+                raise ImageCollectionError(e.obj) from e
 
     if operators and domain is None:
         domain = domain_space.empty()
@@ -100,7 +100,7 @@ def estimate_image(operators=(), vectors=(),
         try:
             rules.apply(op)
         except NoMatchingRuleError as e:
-            raise ImageCollectionError(e.obj)
+            raise ImageCollectionError(e.obj) from e
 
     if riesz_representatives and product:
         image = product.apply_inverse(image)
