@@ -1,6 +1,6 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import scipy.linalg as spla
 import numpy as np
@@ -61,7 +61,7 @@ def solve_ricc_lrcf(A, E, B, C, R=None, trans=False, options=None):
     See :func:`pymor.algorithms.riccati.solve_ricc_lrcf` for a
     general description.
 
-    This function is an implementation of Algorithm 2 in [BBKS18]_.
+    This function is an implementation of Algorithm 2 in :cite:`BBKS18`.
 
     Parameters
     ----------
@@ -74,7 +74,7 @@ def solve_ricc_lrcf(A, E, B, C, R=None, trans=False, options=None):
     C
         The operator C as a |VectorArray| from `A.source`.
     R
-        The operator R as a 2D |NumPy array| or `None`.
+        The matrix R as a 2D |NumPy array| or `None`.
     trans
         Whether the first |Operator| in the Riccati equation is
         transposed.
@@ -88,7 +88,6 @@ def solve_ricc_lrcf(A, E, B, C, R=None, trans=False, options=None):
         Low-rank Cholesky factor of the Riccati equation solution,
         |VectorArray| from `A.source`.
     """
-
     _solve_ricc_check_args(A, E, B, C, R, trans)
     options = _parse_options(options, ricc_lrcf_solver_options(), 'lrradi', None, False)
     logger = getLogger('pymor.algorithms.lrradi.solve_ricc_lrcf')
@@ -184,7 +183,10 @@ def solve_ricc_lrcf(A, E, B, C, R=None, trans=False, options=None):
                 - (F2 @ F2.T) / (4 * shifts[j_shift].real)  \
                 - (F3 @ F3.T) / 2
             Y = spla.block_diag(Y, Yt)
-            EVYt = E.apply(cat_arrays([V.real, V.imag])).lincomb(np.linalg.inv(Yt))
+            if not trans:
+                EVYt = E.apply(cat_arrays([V.real, V.imag])).lincomb(np.linalg.inv(Yt))
+            else:
+                EVYt = E.apply_adjoint(cat_arrays([V.real, V.imag])).lincomb(np.linalg.inv(Yt))
             RF.axpy(np.sqrt(-2 * shifts[j_shift].real), EVYt[:len(C)])
             K += EVYt.lincomb(F2.T)
             j += 2
@@ -207,7 +209,7 @@ def hamiltonian_shifts_init(A, E, B, C, shift_options):
     eigenvalue of the projected Hamiltonian with the most impact on convergence as the next shift
     parameter.
 
-    See [BBKS18]_, pp. 318-321.
+    See :cite:`BBKS18`, pp. 318-321.
 
     Parameters
     ----------
@@ -277,7 +279,7 @@ def hamiltonian_shifts(A, E, B, R, K, Z, shift_options):
     :math:`Z` and return the eigenvalue of the projected Hamiltonian with the most impact on
     convergence as the next shift parameter.
 
-    See [BBKS18]_, pp. 318-321.
+    See :cite:`BBKS18`, pp. 318-321.
 
     Parameters
     ----------
@@ -301,10 +303,9 @@ def hamiltonian_shifts(A, E, B, R, K, Z, shift_options):
     shifts
         A |NumPy array| containing a set of stable shift parameters.
     """
-
     l = shift_options['subspace_columns']
     # always use multiple of len(R) columns
-    l = l // len(R) * len(R)
+    l = max(1, l // len(R)) * len(R)
     if len(Z) < l:
         l = len(Z)
 
