@@ -1,6 +1,6 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 from collections.abc import Iterable, Mapping
 from collections import OrderedDict
@@ -9,6 +9,7 @@ from weakref import WeakValueDictionary
 from pymor.core.base import BasicObject, UberMeta, abstractmethod, classinstancemethod
 from pymor.core.exceptions import NoMatchingRuleError, RuleNotMatchingError
 from pymor.operators.interface import Operator
+from pymor.tools.formatsrc import format_source, print_source
 from pymor.tools.table import format_table
 
 
@@ -56,9 +57,11 @@ class rule:
     condition_description = None
     condition_type = None
 
+    def _ipython_display_(self):
+        print_source(self.action)
+
     def __repr__(self):
-        from pymor.tools.formatsrc import source_repr
-        return source_repr(self.action)
+        return format_source(self.action)
 
     @property
     def action_description(self):
@@ -145,6 +148,7 @@ class match_generic(rule):
 
 class RuleTableMeta(UberMeta):
     """Meta class for |RuleTable|."""
+
     def __new__(cls, name, parents, dct):
         assert 'rules' not in dct
         rules = []
@@ -156,7 +160,8 @@ class RuleTableMeta(UberMeta):
                     raise ValueError('Rule definition names have to start with "action_"')
                 v.name = k
                 rules.append(v)
-        # note: since Python 3.6, the definition order is preserved in dct, so rules has the right order
+        # note: since Python 3.6, the definition order is preserved in dct,
+        # so rules has the right order
         dct['rules'] = rules
         dct['_breakpoint_for_obj'] = WeakValueDictionary()
         dct['_breakpoint_for_name'] = set()
@@ -208,6 +213,7 @@ class RuleTable(BasicObject, metaclass=RuleTableMeta):
 
     @insert_rule.instancemethod
     def insert_rule(self, index, rule_):
+        """:noindex:"""
         assert isinstance(rule_, rule)
         self.rules.insert(index, rule_)
 
@@ -218,6 +224,7 @@ class RuleTable(BasicObject, metaclass=RuleTableMeta):
 
     @append_rule.instancemethod
     def append_rule(self, rule_):
+        """:noindex:"""
         assert isinstance(rule_, rule)
         self.rules.append(rule_)
 
@@ -336,9 +343,9 @@ class RuleTable(BasicObject, metaclass=RuleTableMeta):
         result = {}
         for child in children:
             c = getattr(obj, child)
-            if isinstance(c, Mapping):
+            if isinstance(c, dict):
                 result[child] = {k: self.apply(v) if v is not None else v for k, v in c.items()}
-            elif isinstance(c, Iterable):
+            elif isinstance(c, (list, tuple)):
                 result[child] = tuple(self.apply(v) if v is not None else v for v in c)
             else:
                 result[child] = self.apply(c) if c is not None else c
@@ -361,18 +368,17 @@ class RuleTable(BasicObject, metaclass=RuleTableMeta):
         attributes `a`, for which one of the folling is true:
 
             1. `a` is an |Operator|.
-            2. `a` is a `mapping` and each of its values is either an |Operator| or `None`.
-            3. `a` is an `iterable` and each of its elements is either an |Operator| or `None`.
+            2. `a` is a dict` and each of its values is either an |Operator| or `None`.
+            3. `a` is a `list` or `tuple` and each of its elements is either an |Operator|
+               or `None`.
         """
         children = set()
         for k in obj._init_arguments:
             try:
                 v = getattr(obj, k)
                 if (isinstance(v, Operator)
-                    or isinstance(v, Mapping) and all(isinstance(vv, Operator)
-                                                      or vv is None for vv in v.values())
-                    or isinstance(v, Iterable) and type(v) is not str and all(isinstance(vv, Operator)
-                                                                              or vv is None for vv in v)):
+                        or isinstance(v, dict) and all(isinstance(vv, Operator) or vv is None for vv in v.values())
+                        or isinstance(v, (list, tuple)) and all(isinstance(vv, Operator) or vv is None for vv in v)):
                     children.add(k)
             except AttributeError:
                 pass
