@@ -1,18 +1,22 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import numpy as np
 import scipy.linalg as spla
 import scipy.sparse as sps
 import scipy.sparse.linalg as spsla
 
-from pymor.algorithms.rules import RuleTable, match_class
+from pymor.algorithms.rules import RuleTable, match_class, match_always
+from pymor.core.config import config
+from pymor.core.exceptions import RuleNotMatchingError
 from pymor.operators.block import BlockOperatorBase
 from pymor.operators.constructions import (AdjointOperator, ComponentProjectionOperator, ConcatenationOperator,
                                            IdentityOperator, LincombOperator, LowRankOperator, LowRankUpdatedOperator,
-                                           VectorArrayOperator, ZeroOperator)
-from pymor.operators.numpy import NumpyMatrixOperator
+                                           NumpyConversionOperator, VectorArrayOperator, ZeroOperator)
+from pymor.operators.interface import as_array_max_length
+from pymor.operators.numpy import NumpyHankelOperator, NumpyMatrixOperator
+from pymor.vectorarrays.numpy import NumpyVectorSpace
 
 
 def to_matrix(op, format=None, mu=None):
@@ -36,6 +40,7 @@ def to_matrix(op, format=None, mu=None):
         The matrix equivalent to `op`.
     """
     assert format is None or format in ('dense', 'bsr', 'coo', 'csc', 'csr', 'dia', 'dok', 'lil')
+    assert op.linear
     op = op.assemble(mu)
     return ToMatrixRules(format, mu).apply(op)
 
@@ -46,6 +51,22 @@ class ToMatrixRules(RuleTable):
         super().__init__()
         self.__auto_init(locals())
 
+    @match_class(NumpyHankelOperator)
+    def action_NumpyHankelOperator(self, op):
+        format = self.format
+        if format in (None, 'dense'):
+            s, p, m = op.markov_parameters.shape
+            n = s // 2 + 1
+            op_mp = np.concatenate([op.markov_parameters, np.zeros([1 - s % 2, p, m])])
+            r, c = op_mp[:n], op_mp[n - 1:]
+            op_matrix = np.zeros([p * n, m * n], dtype=op_mp.dtype)
+            for (i, j) in np.ndindex((p, m)):
+                op_matrix[i::p, j::m] = spla.hankel(r[:, i, j], c[:, i, j])
+
+            return op_matrix
+        else:
+            raise NotImplementedError
+
     @match_class(NumpyMatrixOperator)
     def action_NumpyMatrixOperator(self, op):
         format = self.format
@@ -172,3 +193,41 @@ class ToMatrixRules(RuleTable):
             return np.zeros((op.range.dim, op.source.dim))
         else:
             return getattr(sps, format + '_matrix')((op.range.dim, op.source.dim))
+
+    @match_always
+    def action_as_array(self, op):
+        if op.source.dim < as_array_max_length():
+            if not isinstance(op.source, NumpyVectorSpace):
+                op = op @ NumpyConversionOperator(op.source, 'from_numpy')
+            try:
+                return op.as_range_array(mu=self.mu).to_numpy().T
+            except NotImplementedError:
+                pass
+        if op.range.dim < as_array_max_length():
+            if not isinstance(op.range, NumpyVectorSpace):
+                op = NumpyConversionOperator(op.source, 'to_numpy') @ op
+            try:
+                return op.as_source_array(mu=self.mu).to_numpy()
+            except NotImplementedError:
+                pass
+        raise RuleNotMatchingError
+
+
+if config.HAVE_DUNEGDT:
+    from scipy.sparse import csr_matrix
+
+    from pymor.bindings.dunegdt import DuneXTMatrixOperator
+
+    @match_class(DuneXTMatrixOperator)
+    def action_DuneXTMatrixOperator(self, op):
+        data, row_ind, col_ind = op.matrix.to_csr()
+        mat = csr_matrix((data, (row_ind, col_ind)), shape=(op.matrix.rows, op.matrix.cols))
+        format = self.format
+        if format is None:
+            return mat
+        elif format == 'dense':
+            return mat.toarray()
+        else:
+            return mat.asformat(format)
+
+    ToMatrixRules.insert_rule(0, action_DuneXTMatrixOperator)
