@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import numpy as np
 
@@ -24,8 +24,9 @@ class OnedGrid(GridWithOrthogonalCenters):
     reference_element = line
 
     def __init__(self, domain=(0, 1), num_intervals=4, identify_left_right=False):
-        assert domain[0] < domain[1]
         domain = np.array(domain)
+        assert domain.ndim == 1
+        assert domain[0] < domain[1]
         self.__auto_init(locals())
         self._sizes = [num_intervals, num_intervals] if identify_left_right else [num_intervals, num_intervals + 1]
         self._width = np.abs(self.domain[1] - self.domain[0]) / self.num_intervals
@@ -71,7 +72,7 @@ class OnedGrid(GridWithOrthogonalCenters):
     def orthogonal_centers(self):
         return self.centers(0)
 
-    def visualize(self, U, codim=2, **kwargs):
+    def visualize(self, U, codim=1, **kwargs):
         """Visualize scalar data associated to the grid as a patch plot.
 
         Parameters
@@ -82,18 +83,18 @@ class OnedGrid(GridWithOrthogonalCenters):
             |Numpy arrays| can be provided, in which case a subplot is created for
             each entry of the tuple. The lengths of all arrays have to agree.
         codim
-            The codimension of the entities the data in `U` is attached to (either 0 or 2).
+            The codimension of the entities the data in `U` is attached to (either 0 or 1).
         kwargs
-            See :func:`~pymor.discretizers.builtin.gui.qt.visualize_patch`
+            See :func:`~pymor.discretizers.builtin.gui.visualizers.OnedVisualizer.visualize`
         """
-        from pymor.discretizers.builtin.gui.qt import visualize_matplotlib_1d
+        from pymor.discretizers.builtin.gui.visualizers import OnedVisualizer
         from pymor.vectorarrays.interface import VectorArray
-        from pymor.vectorarrays.numpy import NumpyVectorSpace, NumpyVectorArray
+        from pymor.vectorarrays.numpy import NumpyVectorSpace
         if isinstance(U, (np.ndarray, VectorArray)):
             U = (U,)
         assert all(isinstance(u, (np.ndarray, VectorArray)) for u in U)
         U = tuple(NumpyVectorSpace.make_array(u) if isinstance(u, np.ndarray) else
-                  u if isinstance(u, NumpyVectorArray) else
+                  u if isinstance(u.space, NumpyVectorSpace) else
                   NumpyVectorSpace.make_array(u.to_numpy())
                   for u in U)
-        visualize_matplotlib_1d(self, U, codim=codim, **kwargs)
+        OnedVisualizer(self, codim=codim).visualize(U, **kwargs)
