@@ -1,6 +1,6 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import weakref
 
@@ -72,7 +72,14 @@ class SubGrid(Grid):
         return self.__parent_indices[codim]
 
     def indices_from_parent_indices(self, ind, codim):
-        """Maps a |NumPy array| of indicies of codim-`codim` entites of the parent grid to indicies of the subgrid.
+        """Maps indices of codim-`codim` entities of the parent grid to indices of the subgrid.
+
+        Parameters
+        ----------
+        ind
+            |NumPy array| of indices of codim-`codim` entities of the parent grid
+        codim
+            codim of the entities indicated by `ind`
 
         Raises
         ------
@@ -116,7 +123,8 @@ def make_sub_grid_boundary_info(sub_grid, parent_grid, parent_grid_boundary_info
     Parameters
     ----------
     sub_grid
-        The :class:`~pymor.discretizers.builtin.grids.subgrid.SubGrid` for which a |BoundaryInfo| is created.
+        The :class:`~pymor.discretizers.builtin.grids.subgrid.SubGrid` for which a
+        |BoundaryInfo| is created.
     parent_grid
         The parent |Grid|.
     parent_grid_boundary_info
