@@ -1,5 +1,5 @@
 # This file is part of the pyMOR project (https://www.pymor.org).
-# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# Copyright pyMOR developers and contributors. All rights reserved.
 # License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import numpy as np
@@ -231,12 +231,12 @@ class TriaGrid(GridWithOrthogonalCenters):
         """
         from pymor.discretizers.builtin.gui.visualizers import PatchVisualizer
         from pymor.vectorarrays.interface import VectorArray
-        from pymor.vectorarrays.numpy import NumpyVectorSpace, NumpyVectorArray
+        from pymor.vectorarrays.numpy import NumpyVectorSpace
         if isinstance(U, (np.ndarray, VectorArray)):
             U = (U,)
         assert all(isinstance(u, (np.ndarray, VectorArray)) for u in U)
         U = tuple(NumpyVectorSpace.make_array(u) if isinstance(u, np.ndarray) else
-                  u if isinstance(u, NumpyVectorArray) else
+                  u if isinstance(u.space, NumpyVectorSpace) else
                   NumpyVectorSpace.make_array(u.to_numpy())
                   for u in U)
         PatchVisualizer(self, codim=codim).visualize(U, **kwargs)
