@@ -1,6 +1,6 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 """Module containing some constructions to obtain new operators from old ones."""
 
@@ -138,31 +138,32 @@ class LincombOperator(Operator):
         from pymor.algorithms.lincomb import assemble_lincomb
         operators = tuple(op.assemble(mu) for op in self.operators)
         coefficients = self.evaluate_coefficients(mu)
+        # try to form a linear combination
         op = assemble_lincomb(operators, coefficients, solver_options=self.solver_options,
                               name=self.name + '_assembled')
-        if op:
+        # To avoid infinite recursions, only use the result if at least one of the following
+        # is true:
+        #   - The operator is parametric, so the the result of assemble *must* be a different,
+        #     non-parametric operator.
+        #   - One of self.operators changed by calling 'assemble' on it.
+        #   - The result of assemble_lincomb is of a different type than the original operator.
+        #   - assemble_lincomb could simplify the list of assembled operators,
+        #     which we define to be the case when the number of operators has ben reduced.
+        if (self.parametric
+                or operators != self.operators  # for this comparison to work self.operators always has to be a tuple!
+                or type(op) != type(self)
+                or len(op.operators) < len(operators)):
             return op
         else:
-            if self.parametric or operators != self.operators:
-                return LincombOperator(operators, coefficients, solver_options=self.solver_options,
-                                       name=self.name + '_assembled')
-            else:  # this can only happen when both operators and self.operators are tuples!
-                return self  # avoid infinite recursion
+            return self
 
     def jacobian(self, U, mu=None):
-        from pymor.algorithms.lincomb import assemble_lincomb
         if self.linear:
             return self.assemble(mu)
         jacobians = [op.jacobian(U, mu) for op in self.operators]
-        coefficients = self.evaluate_coefficients(mu)
         options = self.solver_options.get('jacobian') if self.solver_options else None
-        jac = assemble_lincomb(jacobians, coefficients, solver_options=options,
-                               name=self.name + '_jacobian')
-        if jac is None:
-            return LincombOperator(jacobians, coefficients, solver_options=options,
-                                   name=self.name + '_jacobian')
-        else:
-            return jac
+        return LincombOperator(jacobians, self.coefficients, solver_options=options,
+                               name=self.name + '_jacobian').assemble(mu)
 
     def d_mu(self, parameter, index=0):
         for op in self.operators:
@@ -178,21 +179,24 @@ class LincombOperator(Operator):
 
     def apply_inverse(self, V, mu=None, initial_guess=None, least_squares=False):
         if len(self.operators) == 1:
-            if self.coefficients[0] == 0.:
+            coeff = self.evaluate_coefficients(mu)[0]
+            if not coeff:
                 if least_squares:
                     return self.source.zeros(len(V))
                 else:
                     raise InversionError
             else:
                 U = self.operators[0].apply_inverse(V, mu=mu, initial_guess=initial_guess, least_squares=least_squares)
-                U *= (1. / self.coefficients[0])
+                coefficients = self.evaluate_coefficients(mu)
+                U *= (1. / coefficients[0])
                 return U
         else:
             return super().apply_inverse(V, mu=mu, initial_guess=initial_guess, least_squares=least_squares)
 
     def apply_inverse_adjoint(self, U, mu=None, initial_guess=None, least_squares=False):
         if len(self.operators) == 1:
-            if self.coefficients[0] == 0.:
+            coeff = self.evaluate_coefficients(mu)[0]
+            if not coeff:
                 if least_squares:
                     return self.range.zeros(len(U))
                 else:
@@ -200,7 +204,7 @@ class LincombOperator(Operator):
             else:
                 V = self.operators[0].apply_inverse_adjoint(U, mu=mu,
                                                             initial_guess=initial_guess, least_squares=least_squares)
-                V *= (1. / self.coefficients[0])
+                V *= (1. / coeff)
                 return V
         else:
             return super().apply_inverse_adjoint(U, mu=mu, initial_guess=initial_guess, least_squares=least_squares)
@@ -220,59 +224,8 @@ class LincombOperator(Operator):
     def as_source_array(self, mu=None):
         return self._as_array(True, mu)
 
-    def _add_sub(self, other, sign):
-        if not isinstance(other, Operator):
-            return NotImplemented
 
-        if self.name != 'LincombOperator':
-            if isinstance(other, LincombOperator) and other.name == 'LincombOperator':
-                operators = (self,) + other.operators
-                coefficients = (1.,) + (other.coefficients if sign == 1. else tuple(-c for c in other.coefficients))
-            else:
-                operators, coefficients = (self, other), (1., sign)
-        elif isinstance(other, LincombOperator) and other.name == 'LincombOperator':
-            operators = self.operators + other.operators
-            coefficients = self.coefficients + (other.coefficients if sign == 1.
-                                                else tuple(-c for c in other.coefficients))
-        else:
-            operators, coefficients = self.operators + (other,), self.coefficients + (sign,)
-
-        return LincombOperator(operators, coefficients, solver_options=self.solver_options)
-
-    def _radd_sub(self, other, sign):
-        if not isinstance(other, Operator):
-            return NotImplemented
-
-        # note that 'other' can never be a LincombOperator
-        if self.name != 'LincombOperator':
-            operators, coefficients = (other, self), (1., sign)
-        else:
-            operators = (other,) + self.operators
-            coefficients = (1.,) + (self.coefficients if sign == 1. else tuple(-c for c in self.coefficients))
-
-        return LincombOperator(operators, coefficients, solver_options=other.solver_options)
-
-    def __add__(self, other):
-        return self._add_sub(other, 1.)
-
-    def __sub__(self, other):
-        return self._add_sub(other, -1.)
-
-    def __radd__(self, other):
-        return self._radd_sub(other, 1.)
-
-    def __rsub__(self, other):
-        return self._radd_sub(other, -1.)
-
-    def __mul__(self, other):
-        assert isinstance(other, (Number, ParameterFunctional))
-        if self.name != 'LincombOperator':
-            return LincombOperator((self,), (other,))
-        else:
-            return self.with_(coefficients=tuple(c * other for c in self.coefficients))
-
-
-class Concatenation(Operator):
+class ConcatenationOperator(Operator):
     """|Operator| representing the concatenation of two |Operators|.
 
     Parameters
@@ -322,43 +275,56 @@ class Concatenation(Operator):
         for op in self.operators[:0:-1]:
             Us.append(op.apply(Us[-1], mu=mu))
         options = self.solver_options.get('jacobian') if self.solver_options else None
-        return Concatenation(tuple(op.jacobian(U, mu=mu) for op, U in zip(self.operators, Us[::-1])),
-                             solver_options=options, name=self.name + '_jacobian')
+        return ConcatenationOperator(tuple(op.jacobian(U, mu=mu) for op, U in zip(self.operators, Us[::-1])),
+                                     solver_options=options, name=self.name + '_jacobian')
+
+    def d_mu(self, parameter, index=0):
+        summands = []
+        for i, op in enumerate(self.operators):
+            op_d_mu = op.d_mu(parameter, index)
+            if isinstance(op_d_mu, ZeroOperator):
+                continue
+            if not all(o.linear for o in self.operators[:i]):
+                raise NotImplementedError
+            ops = list(self.operators)
+            ops[i] = op_d_mu
+            summands.append(ConcatenationOperator(ops))
+        return LincombOperator(summands, [1.] * len(summands))
 
     def restricted(self, dofs):
         restricted_ops = []
         for op in self.operators:
             rop, dofs = op.restricted(dofs)
             restricted_ops.append(rop)
-        return Concatenation(restricted_ops), dofs
+        return ConcatenationOperator(restricted_ops), dofs
 
     def __matmul__(self, other):
         if not isinstance(other, Operator):
             return NotImplemented
 
-        if self.name != 'Concatenation':
-            if isinstance(other, Concatenation) and other.name == 'Concatenation':
+        if self.name != 'ConcatenationOperator':
+            if isinstance(other, ConcatenationOperator) and other.name == 'ConcatenationOperator':
                 operators = (self,) + other.operators
             else:
                 operators = (self, other)
-        elif isinstance(other, Concatenation) and other.name == 'Concatenation':
+        elif isinstance(other, ConcatenationOperator) and other.name == 'ConcatenationOperator':
             operators = self.operators + other.operators
         else:
             operators = self.operators + (other,)
 
-        return Concatenation(operators, solver_options=self.solver_options)
+        return ConcatenationOperator(operators, solver_options=self.solver_options)
 
     def __rmatmul__(self, other):
         if not isinstance(other, Operator):
             return NotImplemented
 
-        # note that 'other' can never be a Concatenation
-        if self.name != 'Concatenation':
+        # note that 'other' can never be a ConcatenationOperator
+        if self.name != 'ConcatenationOperator':
             operators = (other, self)
         else:
             operators = (other,) + self.operators
 
-        return Concatenation(operators, solver_options=other.solver_options)
+        return ConcatenationOperator(operators, solver_options=other.solver_options)
 
 
 class ProjectedOperator(Operator):
@@ -472,7 +438,7 @@ class ProjectedOperator(Operator):
             V = self.range_basis.lincomb(V.to_numpy())
         U = self.operator.apply_adjoint(V, mu)
         if self.source_basis is not None:
-            U = self.source.make_array(U.dot(self.source_basis))
+            U = self.source.make_array(U.inner(self.source_basis))
         return U
 
 
@@ -528,7 +494,7 @@ class LowRankOperator(Operator):
 
     def apply(self, U, mu=None):
         assert U in self.source
-        V = self.right.dot(U)
+        V = self.right.inner(U)
         if self.inverted:
             V = spla.solve(self.core, V)
         else:
@@ -537,7 +503,7 @@ class LowRankOperator(Operator):
 
     def apply_adjoint(self, V, mu=None):
         assert V in self.range
-        U = self.left.dot(V)
+        U = self.left.inner(V)
         if self.inverted:
             U = spla.solve(self.core.T.conj(), U)
         else:
@@ -600,8 +566,8 @@ class LowRankUpdatedOperator(LincombOperator):
         alpha, beta = self.evaluate_coefficients(mu)
         AinvV = A.apply_inverse(V)
         AinvL = A.apply_inverse(L)
-        mat = alpha * C + beta * R.dot(AinvL)
-        RhAinvV = R.dot(AinvV)
+        mat = alpha * C + beta * R.inner(AinvL)
+        RhAinvV = R.inner(AinvV)
         U = AinvV
         U.axpy(-beta, AinvL.lincomb(spla.solve(mat, RhAinvV).T))
         U.scal(1 / alpha)
@@ -618,15 +584,15 @@ class LowRankUpdatedOperator(LincombOperator):
         alpha, beta = (c.conjugate() for c in self.evaluate_coefficients(mu))
         AinvhU = A.apply_inverse_adjoint(U)
         AinvhR = A.apply_inverse_adjoint(R)
-        mat = alpha * C.T.conj() + beta * L.dot(AinvhR)
-        LhAinvhU = L.dot(AinvhU)
+        mat = alpha * C.T.conj() + beta * L.inner(AinvhR)
+        LhAinvhU = L.inner(AinvhU)
         V = AinvhU
         V.axpy(-beta, AinvhR.lincomb(spla.solve(mat, LhAinvhU).T))
         V.scal(1 / alpha)
         return V
 
 
-class ComponentProjection(Operator):
+class ComponentProjectionOperator(Operator):
     """|Operator| representing the projection of a |VectorArray| onto some of its components.
 
     Parameters
@@ -852,7 +818,7 @@ class VectorArrayOperator(Operator):
         if not self.adjoint:
             return self.array.lincomb(U.to_numpy())
         else:
-            return self.range.make_array(self.array.dot(U).T)
+            return self.range.make_array(self.array.inner(U).T)
 
     def apply_inverse(self, V, mu=None, initial_guess=None, least_squares=False):
         if not least_squares and len(self.array) != self.array.dim:
@@ -863,11 +829,11 @@ class VectorArrayOperator(Operator):
 
         Q, R = gram_schmidt(self.array, return_R=True, reiterate=False)
         if self.adjoint:
-            v = lstsq(R.T.conj(), V.to_numpy().T)[0]
+            v = lstsq(R.T.conj(), V.to_numpy().T, rcond=None)[0]
             U = Q.lincomb(v.T)
         else:
-            v = Q.dot(V)
-            u = lstsq(R, v)[0]
+            v = Q.inner(V)
+            u = lstsq(R, v, rcond=None)[0]
             U = self.source.make_array(u.T)
 
         return U
@@ -875,7 +841,7 @@ class VectorArrayOperator(Operator):
     def apply_adjoint(self, V, mu=None):
         assert V in self.range
         if not self.adjoint:
-            return self.source.make_array(self.array.dot(V).T)
+            return self.source.make_array(self.array.inner(V).T)
         else:
             return self.array.lincomb(V.to_numpy())
 
@@ -1069,7 +1035,7 @@ class LinearOperator(ProxyOperator):
 
 
 class AffineOperator(ProxyOperator):
-    """Decompose an affine |Operator| into affine_shift and linear_part. """
+    """Decompose an affine |Operator| into affine_shift and linear_part."""
 
     def __init__(self, operator, name=None):
         if operator.parametric:
@@ -1300,8 +1266,8 @@ class SelectionOperator(Operator):
         The interval boundaries as defined above.
     name
         Name of the operator.
-
     """
+
     def __init__(self, operators, parameter_functional, boundaries, name=None):
         assert len(operators) > 0
         assert len(boundaries) == len(operators) - 1
@@ -1409,3 +1375,58 @@ class InducedNorm(ParametricObject):
         if self.raise_negative and np.any(norm_squared < 0):
             raise ValueError(f'norm is negative (square = {norm_squared})')
         return np.sqrt(norm_squared)
+
+
+class NumpyConversionOperator(Operator):
+    """Converts |VectorArrays| to |NumpyVectorArrays|.
+
+    Note that the input |VectorArrays| need to support
+    :meth:`~pymor.vectorarrays.interface.VectorArray.to_numpy`.
+    For the adjoint,
+    :meth:`~pymor.vectorarrays.interface.VectorSpace.from_numpy`
+    needs to be implemented.
+
+    Parameters
+    ----------
+    space
+        The |VectorSpace| of the |VectorArrays| that are converted to
+        |NumpyVectorArrays|.
+    direction
+        Either `'to_numpy'` or `'from_numpy'`. In case of `'to_numpy'`
+        :meth:`apply` takes a |VectorArray| from `space` and returns
+        a |NumpyVectorArray|. In case of `'from_numpy'`, :meth:`apply`
+        takes a |NumpyVectorArray| and returns a |VectorArray| from
+        `space`.
+    """
+
+    linear = True
+
+    def __init__(self, space, direction='to_numpy'):
+        assert direction in ('to_numpy', 'from_numpy')
+        self.__auto_init(locals())
+        if direction == 'to_numpy':
+            self.source = space
+            self.range = NumpyVectorSpace(space.dim, id=space.id)
+        else:
+            self.source = NumpyVectorSpace(space.dim, id=space.id)
+            self.range = space
+
+    @property
+    def H(self):
+        return self.with_(direction='from_numpy' if self.direction == 'to_numpy' else 'to_numpy')
+
+    def apply(self, U, mu=None):
+        assert U in self.source
+        return self.range.from_numpy(U.to_numpy())
+
+    def apply_inverse(self, V, mu=None, initial_guess=None, least_squares=False):
+        assert V in self.range
+        return self.source.from_numpy(V.to_numpy())
+
+    def apply_adjoint(self, V, mu=None):
+        assert V in self.range
+        return self.source.from_numpy(V.to_numpy())
+
+    def apply_inverse_adjoint(self, U, mu=None, initial_guess=None, least_squares=False):
+        assert U in self.source
+        return self.range.from_numpy(U.to_numpy())
