@@ -1,6 +1,6 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 from numbers import Number
 
@@ -103,7 +103,7 @@ class Operator(ParametricObject):
         V
             |VectorArray| of the left arguments V.
         U
-            |VectorArray| of the right right arguments U.
+            |VectorArray| of the right arguments U.
         mu
             The |parameter values| for which to evaluate the operator.
 
@@ -127,7 +127,7 @@ class Operator(ParametricObject):
 
             op.apply2(V, U)[i] = V[i]^T*M*U[i].
 
-        In the case of complex numbers, note that `pairwirse_apply2` is anti-linear in the
+        In the case of complex numbers, note that `pairwise_apply2` is anti-linear in the
         first variable by definition of `pairwise_inner`.
 
 
@@ -136,7 +136,7 @@ class Operator(ParametricObject):
         V
             |VectorArray| of the left arguments V.
         U
-            |VectorArray| of the right right arguments U.
+            |VectorArray| of the right arguments U.
         mu
             The |parameter values| for which to evaluate the operator.
 
@@ -221,25 +221,43 @@ class Operator(ParametricObject):
         assembled_op = self.assemble(mu)
         if assembled_op != self and not isinstance(assembled_op, FixedParameterOperator):
             return assembled_op.apply_inverse(V, initial_guess=initial_guess, least_squares=least_squares)
-        elif self.linear:
-            options = self.solver_options.get('inverse') if self.solver_options else None
+
+        options = self.solver_options.get('inverse') if self.solver_options else None
+        options = (None if options is None else
+                   {'type': options} if isinstance(options, str) else
+                   options.copy())
+        solver_type = None if options is None else options['type']
+
+        if self.linear:
+            if solver_type is None or solver_type == 'to_matrix':
+                if solver_type is None:
+                    self.logger.warning(f'No specialized linear solver available for {self}.')
+                    self.logger.warning('Trying to solve by converting to NumPy matrix.')
+                from pymor.algorithms.rules import NoMatchingRuleError
+                try:
+                    from pymor.algorithms.to_matrix import to_matrix
+                    from pymor.operators.numpy import NumpyMatrixOperator
+                    mat = to_matrix(assembled_op)
+                    mat_op = NumpyMatrixOperator(mat)
+                    v = mat_op.range.from_numpy(V.to_numpy())
+                    i = None if initial_guess is None else v.source.from_numpy(initial_guess.to_numpy())
+                    u = mat_op.apply_inverse(v, initial_guess=i, least_squares=least_squares)
+                    return self.source.from_numpy(u.to_numpy())
+                except (NoMatchingRuleError, NotImplementedError):
+                    if solver_type == 'to_matrix':
+                        raise InversionError
+                    else:
+                        self.logger.warning('Failed.')
+            self.logger.warning('Solving with unpreconditioned iterative solver.')
             return genericsolvers.apply_inverse(assembled_op, V, initial_guess=initial_guess,
                                                 options=options, least_squares=least_squares)
         else:
             from pymor.algorithms.newton import newton
             from pymor.core.exceptions import NewtonError
 
-            options = self.solver_options.get('inverse') if self.solver_options else None
-            if options:
-                if isinstance(options, str):
-                    assert options == 'newton'
-                    options = {}
-                else:
-                    assert options['type'] == 'newton'
-                    options = options.copy()
-                    options.pop('type')
-            else:
-                options = {}
+            assert solver_type is None or solver_type == 'newton'
+            options = options or {}
+            options.pop('type', None)
             options['least_squares'] = least_squares
 
             with self.logger.block('Solving nonlinear problem using newton algorithm ...'):
@@ -251,7 +269,7 @@ class Operator(ParametricObject):
                                         mu=mu,
                                         **options)[0])
                     except NewtonError as e:
-                        raise InversionError(e)
+                        raise InversionError(e) from e
             return R
 
     def apply_inverse_adjoint(self, U, mu=None, initial_guess=None, least_squares=False):
@@ -545,8 +563,8 @@ class Operator(ParametricObject):
     def _radd_sub(self, other, sign):
         if other == 0:
             return self
-        if not isinstance(other, Operator):
-            return NotImplemented
+        assert not isinstance(other, Operator)  # this is always handled by _add_sub
+        return NotImplemented
 
     def __add__(self, other):
         return self._add_sub(other, 1.)
