@@ -1,5 +1,5 @@
 # This file is part of the pyMOR project (https://www.pymor.org).
-# Copyright 2013-2021 pyMOR developers and contributors. All rights reserved.
+# Copyright pyMOR developers and contributors. All rights reserved.
 # License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 """IRKA-type reductor for |SecondOrderModels|."""
@@ -88,7 +88,7 @@ class SORIRKAReductor(GenericIRKAReductor):
         rom
             Reduced-order |SecondOrderModel|.
         """
-        if not self.fom.cont_time:
+        if self.fom.sampling_time > 0:
             raise NotImplementedError
 
         self._clear_lists()
