@@ -1,6 +1,6 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 from pymor.core.defaults import defaults
 
@@ -9,7 +9,7 @@ import numpy as np
 
 @defaults('compact_print')
 def format_array(array, compact_print=False):
-    '''Creates a formatted string representation of a |NumPy array|.
+    """Creates a formatted string representation of a |NumPy array|.
 
     Parameters
     ----------
@@ -21,7 +21,7 @@ def format_array(array, compact_print=False):
     Returns
     -------
     The string representation.
-    '''
+    """
     def format_element(e):
         if e > 1e15:
             return f'{e:.2e}'
