@@ -1,6 +1,6 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import time
 import numpy as np
@@ -10,7 +10,9 @@ from pymor.core.logger import getLogger
 
 
 class Timer:
-    """You can use me as a context manager, plain instance or decorator to time execution
+    """Class for finding code runtime.
+
+    You can use me as a context manager, plain instance or decorator to time execution
     of a code scope::
 
         with Timer() as timer:
@@ -43,10 +45,10 @@ class Timer:
 
     def start(self):
         self.dt = -1
-        self._start = time.process_time()
+        self._start = time.perf_counter()
 
     def stop(self):
-        self.dt = time.process_time() - self._start
+        self.dt = time.perf_counter() - self._start
 
     def __enter__(self):
         self.start()
