@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import numpy as np
 from typer import Argument, Option, run
@@ -29,24 +29,24 @@ def main(
     """Solves the Poisson equation in 2D using pyMOR's builtin discreization toolkit."""
     norm = float(norm) if not norm.lower() in ('h1', 'l2') else norm.lower()
 
-    rhss = [ExpressionFunction('ones(x.shape[:-1]) * 10', 2, ()),
+    rhss = [ExpressionFunction('10', 2),
             LincombFunction(
-                [ExpressionFunction('ones(x.shape[:-1]) * 10', 2, ()), ConstantFunction(1., 2)],
+                [ExpressionFunction('10', 2), ConstantFunction(1., 2)],
                 [ProjectionParameterFunctional('mu'), 0.1])]
 
-    dirichlets = [ExpressionFunction('zeros(x.shape[:-1])', 2, ()),
+    dirichlets = [ExpressionFunction('0', 2),
                   LincombFunction(
-                  [ExpressionFunction('2 * x[..., 0]', 2, ()), ConstantFunction(1., 2)],
+                  [ExpressionFunction('2 * x[0]', 2), ConstantFunction(1., 2)],
                   [ProjectionParameterFunctional('mu'), 0.5])]
 
     neumanns = [None,
                 LincombFunction(
-                    [ExpressionFunction('1 - x[..., 1]', 2, ()), ConstantFunction(1., 2)],
+                    [ExpressionFunction('1 - x[1]', 2), ConstantFunction(1., 2)],
                     [ProjectionParameterFunctional('mu'), 0.5**2])]
 
     robins = [None,
               (LincombFunction(
-                  [ExpressionFunction('x[..., 1]', 2, ()), ConstantFunction(1., 2)],
+                  [ExpressionFunction('x[1]', 2), ConstantFunction(1., 2)],
                   [ProjectionParameterFunctional('mu'), 1]), ConstantFunction(1., 2))]
 
     domains = [RectDomain(),
@@ -62,7 +62,7 @@ def main(
         domain=domain,
         rhs=rhs,
         diffusion=LincombFunction(
-            [ExpressionFunction('1 - x[..., 0]', 2, ()), ExpressionFunction('x[..., 0]', 2, ())],
+            [ExpressionFunction('1 - x[0]', 2), ExpressionFunction('x[0]', 2)],
             [ProjectionParameterFunctional('mu'), 1]
         ),
         dirichlet_data=dirichlet,
