@@ -1,11 +1,10 @@
-# This file is part of the pyMOR project (http://www.pymor.org).
-# Copyright 2013-2020 pyMOR developers and contributors. All rights reserved.
-# License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
 
 import os
-import pymordemos
+import pymordemos  # noqa: F401
 from importlib import import_module
-import sys
 import pytest
 from tempfile import mkdtemp
 import shutil
@@ -43,7 +42,6 @@ DISCRETIZATION_ARGS = (
     ('burgers', ['--num-flux=lax_friedrichs', '0.1']),
     ('burgers', ['--num-flux=engquist_osher', '0.1']),
     ('burgers', ['--num-flux=simplified_engquist_osher', '0.1']),
-    ('linear_optimization', [40, 20]),
     ('parabolic', ['heat', 1]),
     ('parabolic', ['heat', '--rect', 1]),
     ('parabolic', ['heat', '--fv', 1]),
@@ -54,10 +52,13 @@ DISCRETIZATION_ARGS = (
 
 if not parallel:
     DISCRETIZATION_ARGS += (('elliptic_unstructured', [6., 16, 1e-1]),)
-if not is_windows_platform():
-    DISCRETIZATION_ARGS += (('neural_networks', [25, 50, 10]),
+
+NEURAL_NETWORK_ARGS = (
+    ('neural_networks', [15, 20, 3]),
     ('neural_networks_fenics', [15, 3]),
-    ('neural_networks_instationary', [25, 25, 30, 5]))
+    ('neural_networks_instationary', [0, 10, 3, 15, 3]),
+    ('neural_networks_instationary', [1, 15, 3, 20, 3]),
+)
 
 THERMALBLOCK_ARGS = (
     ('thermalblock', ['--plot-solutions', '--plot-err', '--plot-error-sequence', 2, 2, 3, 5]),
@@ -72,30 +73,24 @@ THERMALBLOCK_ARGS = (
 TB_IPYTHON_ARGS = THERMALBLOCK_ARGS[0:2]
 
 THERMALBLOCK_ADAPTIVE_ARGS = (
-    ('thermalblock_adaptive', [10]),
-    ('thermalblock_adaptive', ['--no-visualize-refinement', 10]),
+    ('thermalblock_adaptive', ['--cache-region=memory', '--plot-solutions', '--plot-error-sequence', 10]),
+    ('thermalblock_adaptive', ['--no-visualize-refinement', '--plot-err', 10]),
 )
 
 THERMALBLOCK_SIMPLE_ARGS = (
     ('thermalblock_simple', ['pymor', 'naive', 2, 5, 5]),
     ('thermalblock_simple', ['fenics', 'greedy', 2, 5, 5]),
     ('thermalblock_simple', ['ngsolve', 'pod', 2, 5, 5]),
+    ('thermalblock_simple', ['--', 'pymor_text', 'adaptive_greedy', -1, 3, 3]),
 )
-# Font file loading currently does not work on windows
-if not is_windows_platform():
-    THERMALBLOCK_SIMPLE_ARGS += (('thermalblock_simple', ['--', 'pymor_text', 'adaptive_greedy', -1, 3, 3]),)
 
-if is_macos_platform():
-    # thermalblock_gui requires OpenGL which somehow fails on macos
-    THERMALBLOCK_GUI_ARGS = tuple()
-else:
-    THERMALBLOCK_GUI_ARGS = (
-        ('thermalblock_gui', ['--testing', 2, 2, 3, 5]),
-    )
+THERMALBLOCK_GUI_ARGS = (
+    ('thermalblock_gui', ['--testing', 2, 2, 3, 5]),
+)
 
 BURGERS_EI_ARGS = (
-    ('burgers_ei', [1, 2, 2, 5, 2, 5, '--plot-ei-err']),
-    ('burgers_ei', [1, 2, 2, 5, 2, 5, '--ei-alg=deim']),
+    ('burgers_ei', [1, 2, 2, 5, 2, 5, '--plot-ei-err', '--plot-err', '--plot-solutions']),
+    ('burgers_ei', [1, 2, 2, 5, 2, 5, '--ei-alg=deim', '--plot-error-landscape']),
 )
 
 PARABOLIC_MOR_ARGS = (
@@ -110,14 +105,15 @@ SYS_MOR_ARGS = (
     ('string_equation', [25, 2]),
     ('parametric_heat', [0.02, 2]),
     ('parametric_delay', [2]),
-    ('parametric_string', [25, 2]),
+    ('parametric_string', [5, 2]),
     ('parametric_synthetic', [100, 2]),
+    ('unstable_heat', [50, 10]),
 )
 
 HAPOD_ARGS = (
     ('hapod', ['--snap=3', 1e-2, 10, 100]),
     ('hapod', ['--snap=3', '--threads=2', 1e-2, 10, 100]),
-    ('hapod', ['--snap=3', '--procs=2', 1e-2, 10, 100]),
+    ('hapod', ['--snap=3', '--procs=2', '--arity=2', 1e-2, 10, 100]),
 )
 
 FENICS_NONLINEAR_ARGS = (
@@ -125,8 +121,37 @@ FENICS_NONLINEAR_ARGS = (
     ('fenics_nonlinear', [3, 5, 1]),
 )
 
+FUNCTION_EI_ARGS = (
+    ('function_ei', ['--grid=10', 3, 2, 3, 2, '--plot-ei-err', '--plot-solutions']),
+)
+
+OUTPUT_FUNCTIONAL_ARGS = (
+    ('linear_optimization', [40, 20]),
+    ('output_error_estimation', [0, 10, 4, 10, 0]),
+    ('output_error_estimation', [0, 10, 4, 10, 1]),
+    ('output_error_estimation', [1, 10, 4, 10, 1]),
+    ('output_error_estimation', [2, 10, 4, 10, 0]),
+    ('output_error_estimation', [2, 10, 4, 10, 1]),
+    ('output_error_estimation', [3, 10, 10, 10, 1]),
+    ('output_error_estimation', [4, 10, 10, 10, 1]),
+)
+
+DMD_ARGS = (
+    ('burgers_dmd', [1.5, '--grid=10', '--nt=100']),
+    ('dmd_identification', ['--n=4', '--m=10']),
+)
+
+PHLTI_ARGS = (
+    ('phlti', ['--n=10']),
+)
+
+SYMPLECTIC_WAVE_ARGS = (
+    ('symplectic_wave_equation', [1., 10]),
+)
+
 DEMO_ARGS = (
     DISCRETIZATION_ARGS
+    + NEURAL_NETWORK_ARGS
     + THERMALBLOCK_ARGS
     + THERMALBLOCK_ADAPTIVE_ARGS
     + THERMALBLOCK_SIMPLE_ARGS
@@ -136,28 +161,29 @@ DEMO_ARGS = (
     + SYS_MOR_ARGS
     + HAPOD_ARGS
     + FENICS_NONLINEAR_ARGS
+    + FUNCTION_EI_ARGS
+    + OUTPUT_FUNCTIONAL_ARGS
+    + DMD_ARGS
+    + PHLTI_ARGS
+    + SYMPLECTIC_WAVE_ARGS
 )
+
 DEMO_ARGS = [(f'pymordemos.{a}', b) for (a, b) in DEMO_ARGS]
 
 
 def _skip_if_no_solver(param):
     demo, args = param
     from pymor.core.config import config
-    for solver in ['fenics', 'ngsolve']:
+    for solver, package in [('fenics', None), ('ngsolve', None), ('neural_', 'TORCH'),
+                            ('neural_networks_instationary', 'FENICS')]:
+        package = package or solver.upper()
         needs_solver = len([f for f in args if solver in str(f)]) > 0 or demo.find(solver) >= 0
-        has_solver = getattr(config, 'HAVE_' + solver.upper())
+        has_solver = getattr(config, f'HAVE_{package}')
         if needs_solver and not has_solver:
             if not os.environ.get('DOCKER_PYMOR', False):
                 pytest.skip('skipped test due to missing ' + solver)
 
 
-def _skip_unsupported_torch(param):
-    demo, args = param
-    unsupported_py = sys.version_info[0:2] > (3, 8)
-    if unsupported_py and os.environ.get('DOCKER_PYMOR', False) and 'neural_network' in demo:
-        pytest.skip('skipped test due to torch unsupported on python f{sys.version_info[0:2]}')
-
-
 def _demo_ids(demo_args):
     def _key(b):
         return ' '.join((str(s) for s in b))
@@ -167,7 +193,6 @@ def _demo_ids(demo_args):
 @pytest.fixture(params=DEMO_ARGS, ids=_demo_ids(DEMO_ARGS))
 def demo_args(request):
     _skip_if_no_solver(request.param)
-    _skip_unsupported_torch(request.param)
     return request.param
 
 
@@ -186,12 +211,20 @@ def _test_demo(demo):
 
     try:
         from matplotlib import pyplot
-        pyplot.show = nop
+        if sys.version_info[:2] > (3, 7) or (
+                sys.version_info[0] == 3 and sys.version_info[1] == 6):
+            pyplot.ion()
+        else:
+            # the ion switch results in interpreter segfaults during multiple
+            # demo tests on 3.7 -> fall back on old monkeying solution
+            pyplot.show = nop
     except ImportError:
         pass
     try:
-        import dolfin
-        dolfin.plot = nop
+        import petsc4py
+        # the default X handlers can interfere with process termination
+        petsc4py.PETSc.Sys.popSignalHandler()
+        petsc4py.PETSc.Sys.popErrorHandler()
     except ImportError:
         pass
 
@@ -204,7 +237,7 @@ def _test_demo(demo):
         result = demo()
     except (QtMissing, GmshMissing, MeshioMissing, TorchMissing) as e:
         if os.environ.get('DOCKER_PYMOR', False):
-            # these are all installed in our CI env so them missing a grave error
+            # these are all installed in our CI env so them missing is a grave error
             raise e
         else:
             miss = str(type(e)).replace('Missing', '')
@@ -223,6 +256,9 @@ def _test_demo(demo):
 
 def test_demos(demo_args):
     module, args = demo_args
+    # assertions in pymordemos do not get changed by pytest by default
+    # https://docs.pytest.org/en/stable/writing_plugins.html#assertion-rewriting
+    pytest.register_assert_rewrite(module)
     module = import_module(module)
     if hasattr(module, 'app'):
         app = module.app
@@ -247,7 +283,7 @@ def test_analyze_pickle1():
 def test_analyze_pickle2():
     d = mkdtemp()
     try:
-        test_demos(('pymordemos.thermalblock', ['--pickle=' + os.path.join(d, 'data'), 2, 2, 2, 10]))
+        test_demos(('pymordemos.thermalblock_adaptive', ['--pickle=' + os.path.join(d, 'data'), 10]))
         test_demos(('pymordemos.analyze_pickle',
                    ['histogram', '--detailed-data=' + os.path.join(d, 'data_detailed'), os.path.join(d, 'data_reduced'),
                     10]))
