@@ -1,10 +1,18 @@
+# This file is part of the pyMOR project (https://www.pymor.org).
+# Copyright pyMOR developers and contributors. All rights reserved.
+# License: BSD 2-Clause License (https://opensource.org/licenses/BSD-2-Clause)
+import functools
+
 from hypothesis import strategies as hyst
 from hypothesis import assume, given
 from hypothesis.extra import numpy as hynp
 import numpy as np
 from scipy.stats._multivariate import random_correlation_gen
 
+from pymor.analyticalproblems.functions import Function, ExpressionFunction, ConstantFunction
 from pymor.core.config import config
+from pymor.parameters.base import Mu
+from pymor.tools.deprecated import Deprecated
 from pymor.vectorarrays.list import NumpyListVectorSpace
 from pymor.vectorarrays.block import BlockVectorSpace
 from pymor.vectorarrays.numpy import NumpyVectorSpace
@@ -14,7 +22,10 @@ if config.HAVE_FENICS:
     from pymor.bindings.fenics import FenicsVectorSpace
 
 if config.HAVE_DEALII:
-    from pydealii.pymor.vectorarray import DealIIVectorSpace
+    from pymor_dealii.pymor.vectorarray import DealIIVectorSpace
+
+if config.HAVE_DUNEGDT:
+    from pymor.bindings.dunegdt import DuneXTVectorSpace
 
 if config.HAVE_NGSOLVE:
     import ngsolve as ngs
@@ -23,17 +34,25 @@ if config.HAVE_NGSOLVE:
 
 
 # hypothesis will gladly fill all our RAM with vector arrays if it's not restricted.
-MAX_LENGTH = 102
-hy_lengths = hyst.integers(min_value=0, max_value=MAX_LENGTH)
+MAX_VECTORARRAY_LENGTH = 102
+hy_lengths = hyst.integers(min_value=0, max_value=MAX_VECTORARRAY_LENGTH)
 # this is a legacy restriction, some tests will not work as expected when this is changed/unset
 MAX_ARRAY_ELEMENT_ABSVALUE = 1
-hy_float_array_elements = hyst.floats(allow_nan=False, allow_infinity=False,
-                                      min_value=-MAX_ARRAY_ELEMENT_ABSVALUE, max_value=MAX_ARRAY_ELEMENT_ABSVALUE)
-# the magnitute restriction is also a legacy one
-MAX_COMPLEX_MAGNITUDE = 2
-hy_complex_array_elements = hyst.complex_numbers(allow_nan=False, allow_infinity=False,
-                                                 max_magnitude=MAX_COMPLEX_MAGNITUDE)
+MIN_ARRAY_ELEMENT_ABSVALUE = 1e-34
+
+
 hy_dtypes = hyst.sampled_from([np.float64, np.complex128])
+hy_float_array_elements = hyst.floats(allow_nan=False, allow_infinity=False, allow_subnormal=False,
+                                      min_value=-MAX_ARRAY_ELEMENT_ABSVALUE, max_value=MAX_ARRAY_ELEMENT_ABSVALUE)
+
+
+@Deprecated("hypothesis.strategies.complex_numbers(allow_subnormal=False)")
+@hyst.composite
+def hy_complex_array_elements(draw):
+    # This is a crutch in place for https://github.com/HypothesisWorks/hypothesis/issues/3390
+    parts = hyst.floats(allow_nan=False, allow_infinity=False, allow_subnormal=False,
+                        min_value=-MAX_ARRAY_ELEMENT_ABSVALUE, max_value=MAX_ARRAY_ELEMENT_ABSVALUE)
+    return complex(draw(parts), draw(parts))
 
 
 @hyst.composite
@@ -42,7 +61,7 @@ def _hy_dims(draw, count, compatible):
     if compatible:
         return draw(equal_tuples(dims, count))
     dim_tuple = draw(hyst.tuples(*[dims for _ in range(count)]))
-    for d in range(1,count):
+    for d in range(1, count):
         assume(dim_tuple[d] != dim_tuple[0])
     return dim_tuple
 
@@ -54,9 +73,9 @@ def nothing(*args, **kwargs):
 def _np_arrays(length, dim, dtype=None):
     if dtype is None:
         return hynp.arrays(dtype=np.float64, shape=(length, dim), elements=hy_float_array_elements) | \
-               hynp.arrays(dtype=np.complex128, shape=(length, dim), elements=hy_complex_array_elements)
+            hynp.arrays(dtype=np.complex128, shape=(length, dim), elements=hy_complex_array_elements())
     if dtype is np.complex128:
-        return hynp.arrays(dtype=dtype, shape=(length, dim), elements=hy_complex_array_elements)
+        return hynp.arrays(dtype=dtype, shape=(length, dim), elements=hy_complex_array_elements())
     if dtype is np.float64:
         return hynp.arrays(dtype=dtype, shape=(length, dim), elements=hy_float_array_elements)
     raise RuntimeError(f'unsupported dtype={dtype}')
@@ -94,11 +113,12 @@ def _block_vector_spaces(draw, np_data_list, compatible, count, dims):
         ret.append((BlockVectorSpace(constituent_spaces), ar))
     return ret
 
+
 _other_vector_space_types = []
 
 if config.HAVE_FENICS:
     _FENICS_spaces = {}
-    
+
     def _fenics_vector_spaces(draw, np_data_list, compatible, count, dims):
         ret = []
         for d, ar in zip(dims, np_data_list):
@@ -133,6 +153,11 @@ if config.HAVE_DEALII:
         return [(DealIIVectorSpace(d), ar) for d, ar in zip(dims, np_data_list)]
     _other_vector_space_types.append('dealii')
 
+if config.HAVE_DUNEGDT:
+    def _dunegdt_vector_spaces(draw, np_data_list, compatible, count, dims):
+        return [(DuneXTVectorSpace(d), ar) for d, ar in zip(dims, np_data_list)]
+    _other_vector_space_types.append('dunegdt')
+
 
 _picklable_vector_space_types = ['numpy', 'numpy_list', 'block']
 
@@ -149,7 +174,8 @@ def vector_arrays(draw, space_types, count=1, dtype=None, length=None, compatibl
     assume(len(ret))
     if len(ret) == 1:
         assert count == 1
-        # in test funcs where we only need one array this saves a line to access the single list element
+        # in test funcs where we only need one array this saves a line to access the single list
+        # element
         return ret[0]
     assert count > 1
     return ret
@@ -158,9 +184,10 @@ def vector_arrays(draw, space_types, count=1, dtype=None, length=None, compatibl
 def given_vector_arrays(which='all', count=1, dtype=None, length=None, compatible=True, index_strategy=None, **kwargs):
     """This decorator hides the combination details of given
 
-    the decorated function will be first wrapped in a |hypothesis.given| (with expanded `given_args` and then in
-    |pytest.mark.parametrize| with selected implementation names. The decorated test function must
-    still draw (which a vector_arrays or similar strategy) from the `data` argument in the default case.
+    the decorated function will be first wrapped in a |hypothesis.given| (with expanded `given_args`
+    and then in |pytest.mark.parametrize| with selected implementation names. The decorated test
+    function must still draw (which a vector_arrays or similar strategy) from the `data` argument in
+    the default case.
 
     Parameters
     ----------
@@ -171,18 +198,21 @@ def given_vector_arrays(which='all', count=1, dtype=None, length=None, compatibl
         passed to `given` decorator as is, use for additional strategies
 
     count
-        how many vector arrays to return (in a list), count=1 is special cased to just return the array
+        how many vector arrays to return (in a list), count=1 is special cased to just return the
+        array
     dtype
         dtype of the foundational numpy data the vector array is constructed from
     length
         a hypothesis.strategy how many vectors to generate in each vector array
     compatible
-        if count > 1, this switch toggles generation of vector_arrays with compatible `dim`, `length` and `dtype`
+        if count > 1, this switch toggles generation of vector_arrays with compatible `dim`,
+        `length` and `dtype`
     """
+    @functools.wraps(given)
     def inner_backend_decorator(func):
         try:
             use_imps = {'all': _picklable_vector_space_types  + _other_vector_space_types,
-                           'picklable': _picklable_vector_space_types}[which]
+                        'picklable': _picklable_vector_space_types}[which]
         except KeyError:
             use_imps = which
         first_args = {}
@@ -191,7 +221,8 @@ def given_vector_arrays(which='all', count=1, dtype=None, length=None, compatibl
                 count=count, dtype=dtype, length=length, compatible=compatible, space_types=use_imps))
             first_args['vectors_and_indices'] = arr_ind_strategy
         else:
-            arr_strategy = vector_arrays(count=count, dtype=dtype, length=length, compatible=compatible, space_types=use_imps)
+            arr_strategy = vector_arrays(count=count, dtype=dtype, length=length, compatible=compatible,
+                                         space_types=use_imps)
             if count > 1:
                 first_args['vector_arrays'] = arr_strategy
             else:
@@ -215,13 +246,20 @@ def valid_inds(v, length=None, random_module=None):
         yield list(range(int(len(v)/2)))
         yield list(range(len(v))) * 2
         # TODO what's with the magic number here?
+        # Maybe related to this?
+        # pymortests/vectorarray.py:910: VisibleDeprecationWarning:
+        #   Creating an ndarray from nested sequences exceeding
+        #   the maximum number of dimensions of 32 is deprecated.
+        #   If you mean to do this, you must specify
+        #  'dtype=object' when creating the ndarray.
         length = 32
     if len(v) > 0:
         for ind in [-len(v), 0, len(v) - 1]:
             yield ind
         if len(v) == length:
             yield slice(None)
-        # this avoids managing random state "against" hypothesis when this function is used in a strategy
+        # this avoids managing random state "against" hypothesis when this function is used in a
+        # strategy
         if random_module is None:
             np.random.seed(len(v) * length)
         yield list(np.random.randint(-len(v), len(v), size=length))
@@ -232,9 +270,25 @@ def valid_inds(v, length=None, random_module=None):
 
 
 @hyst.composite
-def valid_indices(draw, array_strategy):
+def valid_indices(draw, array_strategy, length=None):
     v = draw(array_strategy)
-    return v, draw(hyst.sampled_from(list(valid_inds(v))))
+    ints = hyst.integers(min_value=-len(v), max_value=max(len(v)-1, 0))
+    indices = hyst.nothing()
+    if length is None:
+        indices = indices | hyst.just([]) | hyst.lists(ints, max_size=2*len(v))
+    else:
+        indices = indices | hyst.slices(length)
+    if len(v) > 0:
+        inds = [-len(v), 0, len(v) - 1]
+        if len(v) == length:
+            inds.append(slice(None))
+        indices = indices | hyst.lists(ints, max_size=length)
+    else:
+        inds = []
+        if len(v) == 0:
+            inds.append(slice(0, 0))
+    indices = indices | hyst.sampled_from(inds)
+    return v, draw(indices)
 
 
 # TODO match st_valid_inds_of_same_length results to this
@@ -254,7 +308,8 @@ def valid_inds_of_same_length(v1, v2, random_module=None):
         yield -len(v1), -len(v2)
         yield [0], 0
         yield (list(range(min(len(v1), len(v2))//2)),) * 2
-        # this avoids managing random state "against" hypothesis when this function is used in a strategy
+        # this avoids managing random state "against" hypothesis when this function is used in a
+        # strategy
         if random_module is None:
             np.random.seed(len(v1) * len(v2))
         for count in np.linspace(0, min(len(v1), len(v2)), 3).astype(int):
@@ -268,16 +323,33 @@ def valid_inds_of_same_length(v1, v2, random_module=None):
 def st_valid_inds_of_same_length(draw, v1, v2):
     len1, len2 = len(v1), len(v2)
     ret = hyst.just(([], []))
-    # TODO we should include integer arrays here by chaining `| hynp.integer_array_indices(shape=(LEN_X,))`
-    val1 = hynp.basic_indices(shape=(len1,), allow_ellipsis=False)
+    # TODO we should include integer arrays here by chaining
+    # `| hynp.integer_array_indices(shape=(LEN_X,))`
     if len1 == len2:
-        ret = ret | hyst.tuples(hyst.shared(val1, key="st_valid_inds_of_same_length"), hyst.shared(val1, key="st_valid_inds_of_same_length"))
+        ints = hyst.integers(min_value=-len1, max_value=max(len1 - 1, 0))
+        slicer = hyst.slices(len1) | hyst.lists(ints, max_size=len1)
+        ret = ret | hyst.tuples(hyst.shared(slicer, key="st_valid_inds_of_same_length"),
+                                hyst.shared(slicer, key="st_valid_inds_of_same_length"))
     if len1 > 0 and len2 > 0:
-        val2 = hynp.basic_indices(shape=(len2,), allow_ellipsis=False)
-        ret = ret | hyst.tuples(val1, val2)
+        mlen = min(len1, len2)
+        ints = hyst.integers(min_value=-mlen, max_value=max(mlen - 1, 0))
+        slicer = hyst.slices(mlen) | ints | hyst.lists(ints, max_size=mlen)
+        ret = ret | hyst.tuples(hyst.shared(slicer, key="st_valid_inds_of_same_length_uneven"),
+                                hyst.shared(slicer, key="st_valid_inds_of_same_length_uneven"))
     return draw(ret)
 
 
+@hyst.composite
+def st_scaling_value(draw, v1, v2=None):
+    v1 = draw(v1)
+    ints = hyst.integers(min_value=-1, max_value=23)
+    r1 = draw(ints | hyst.just(np.arange(len(v1))))
+    if v2:
+        v2 = draw(v2)
+        return v1, v2, r1, draw(ints | hyst.just(np.arange(len(v2))))
+    return v1, r1
+
+
 # TODO match st_valid_inds_of_different_length results to this
 def valid_inds_of_different_length(v1, v2, random_module):
     # note this potentially yields no result at all for dual 0 length inputs
@@ -293,7 +365,8 @@ def valid_inds_of_different_length(v1, v2, random_module):
         if len(v2) > 1:
             yield 0, [0, 1]
             yield [0], [0, 1]
-        # this avoids managing random state "against" hypothesis when this function is used in a strategy
+        # this avoids managing random state "against" hypothesis when this function is used in a
+        # strategy
         if random_module is None:
             np.random.seed(len(v1) * len(v2))
         for count1 in np.linspace(0, len(v1), 3).astype(int):
@@ -309,49 +382,54 @@ def valid_inds_of_different_length(v1, v2, random_module):
 
 @hyst.composite
 def st_valid_inds_of_different_length(draw, v1, v2):
+    def _filter(x):
+        a, b = x
+        a_type, b_type = type(a), type(b)
+        if a_type != b_type:
+            return True  # tuple + scalar index
+        if a_type == tuple:
+            return len(a) != len(b)
+        return False  # both scalars => not of different length
+
     len1, len2 = len(v1), len(v2)
-    ret = nothing()
     # TODO we should include integer arrays here
-    val = hynp.basic_indices(shape=(len1,), allow_ellipsis=False)  # | hynp.integer_array_indices(shape=(len1,))
-    if len1 != len2:
-        ret = ret | hyst.just((slice(None), slice(None))) \
-              | hyst.tuples(hyst.shared(val, key="indfl"), hyst.shared(val, key="indfl"))
-    if len1 > 0 and len2 > 0:
-        ret = ret | hyst.tuples(val, val).filter(lambda x: len(x[0])!=len(x[1]))
+    val1 = hyst.slices(len1)  # | hynp.integer_array_indices(shape=(len1,))
+    val2 = hyst.slices(len2)  # | hynp.integer_array_indices(shape=(len1,))
+    ret = hyst.tuples(val1, val2).filter(_filter)
     return draw(ret)
 
 
+@hyst.composite
+def st_valid_inds_of_same_or_different_length(draw, v1, v2):
+    return draw(st_valid_inds_of_same_length(v1, v2) | st_valid_inds_of_different_length(v1, v2))
+
+
 @hyst.composite
 def same_and_different_length(draw, array_strategy):
     v = draw(array_strategy)
     if isinstance(v, list):
-        # TODO this should use the st_valid_inds forms directly instead
-        return v, draw(hyst.one_of(hyst.sampled_from(list(valid_inds_of_same_length(*v, random_module=False))),
-                                   hyst.sampled_from(list(valid_inds_of_different_length(*v, random_module=False)))))
-    return v, draw(hyst.one_of(hyst.sampled_from(list(valid_inds_of_same_length(v, v, random_module=False))),
-                               hyst.sampled_from(list(valid_inds_of_different_length(v, v, random_module=False)))))
+        return v, draw(st_valid_inds_of_same_or_different_length(*v))
+    return v, draw(st_valid_inds_of_same_or_different_length(v, v))
 
 
 @hyst.composite
 def pairs_same_length(draw, array_strategy):
     v = draw(array_strategy)
     if isinstance(v, list):
-        # TODO this should use the st_valid_inds forms directly instead
-        return v, draw(hyst.sampled_from(list(valid_inds_of_same_length(*v, random_module=False))))
-    return v, draw(hyst.sampled_from(list(valid_inds_of_same_length(v, v, random_module=False))))
+        return v, draw(st_valid_inds_of_same_length(*v))
+    return v, draw(st_valid_inds_of_same_length(v, v))
 
 
 @hyst.composite
 def pairs_diff_length(draw, array_strategy):
     v = draw(array_strategy)
-    # TODO this should use the st_valid_inds forms directly instead
     if isinstance(v, list):
-        ind_list = list(valid_inds_of_different_length(*v, random_module=False))
+        ind_list = draw(st_valid_inds_of_different_length(*v))
     else:
-        ind_list = list(valid_inds_of_different_length(v, v, random_module=False))
+        ind_list = draw(st_valid_inds_of_different_length(v, v))
     # the consuming tests do not work for None as index
     assume(len(ind_list))
-    return v, draw(hyst.sampled_from(ind_list))
+    return v, ind_list
 
 
 @hyst.composite
@@ -359,9 +437,19 @@ def pairs_both_lengths(draw, array_strategy):
     return draw(hyst.one_of(pairs_same_length(array_strategy), pairs_diff_length(array_strategy)))
 
 
+@hyst.composite
+def invalid_indices(draw, array_strategy):
+    length = 42
+    v = draw(array_strategy)
+    assert not isinstance(v, list)
+    invalid_inds = (None, len(v), [len(v)], -len(v) - 1, [-len(v) - 1], [0, len(v)],
+                    [-len(v) - 1] + [0, ] * (length - 1), list(range(length - 1)) + [len(v)])
+    return v, draw(hyst.sampled_from(invalid_inds))
+
+
 @hyst.composite
 def base_vector_arrays(draw, count=1, dtype=None, max_dim=100):
-    """
+    """Strategy to generate linear independent |VectorArray| inputs for test functions
 
     Parameters
     ----------
@@ -370,9 +458,9 @@ def base_vector_arrays(draw, count=1, dtype=None, max_dim=100):
     dtype dtype for the generated bases, defaults to `np.float_`
     max_dim size limit for the generated
 
-    Returns a list of |VectorArray| linear-independent objects of same dim and length
+    Returns
     -------
-
+    a list of |VectorArray| linear-independent objects of same dim and length
     """
     dtype = dtype or np.float_
     # simplest way currently of getting a |VectorSpace| to construct our new arrays from
@@ -388,7 +476,7 @@ def base_vector_arrays(draw, count=1, dtype=None, max_dim=100):
     random_correlation = random_correlation_gen(random.seed)
 
     def _eigs():
-        """sum must equal to `length` for the scipy construct method"""
+        """Sum must equal to `length` for the scipy construct method"""
         min_eig, max_eig = 0.001, 1.
         eigs = np.asarray((max_eig-min_eig)*np.random.random(length-1) + min_eig, dtype=float)
         return np.append(eigs, [length - np.sum(eigs)])
@@ -397,7 +485,7 @@ def base_vector_arrays(draw, count=1, dtype=None, max_dim=100):
         mat = [random_correlation.rvs(_eigs(), tol=1e-12) for _ in range(count)]
         return [space.from_numpy(m) for m in mat]
     else:
-        scalar = 4*np.random.random((1,1))+0.1
+        scalar = 4*np.random.random((1, 1))+0.1
         return [space.from_numpy(scalar) for _ in range(count)]
 
 
@@ -407,15 +495,15 @@ def equal_tuples(draw, strategy, count):
     return draw(hyst.tuples(*[hyst.just(val) for _ in range(count)]))
 
 
-def invalid_inds(v, length=None):
-    yield None
-    if length is None:
-        yield len(v)
-        yield [len(v)]
-        yield -len(v)-1
-        yield [-len(v)-1]
-        yield [0, len(v)]
-        length = 42
-    if length > 0:
-        yield [-len(v)-1] + [0, ] * (length - 1)
-        yield list(range(length - 1)) + [len(v)]
+# stick to a few representative examples to avoid only seeing degenerate cases
+# in the selected examples
+mus = hyst.dictionaries(
+    keys=hyst.sampled_from(['t', 'foo', 'bar']),
+    values=hyst.sampled_from([
+        np.array([1.]),
+        np.array([1., 32., 3]),
+        ExpressionFunction('x+1', 1),
+        ExpressionFunction('[1., 0] * x + [0, 1.] * x**2', 1),
+        ConstantFunction(np.array([1., 2, 3]))
+    ])
+).filter(lambda mu: 't' not in mu or (not isinstance(mu['t'], Function) and len(mu['t']) == 1)).map(Mu)
